import fs_implementation from 'node:fs';
import process from 'node:process';
import path from 'node:path';

function sync_impl$g(fs, path, mode) {
	fs.chmodSync(path, mode);
}

async function async_impl$g(fs, path, mode) {
	await fs.promises.chmod(path, mode);
}

var chmod = {sync_impl: sync_impl$g, async_impl: async_impl$g};

function sync_impl$f(fs, path, uid, gid) {
	fs.chownSync(path, uid, gid);
}

async function async_impl$f(fs, path, uid, gid) {
	await fs.promises.chown(path, uid, gid);
}

var chown = {sync_impl: sync_impl$f, async_impl: async_impl$f};

function sync_impl$e(fs, src, dest) {
	fs.copyFileSync(src, dest);
}

async function async_impl$e(fs, src, dest) {
	await fs.promises.copyFile(src, dest);
}

var copyFile = {sync_impl: sync_impl$e, async_impl: async_impl$e};

function sync_impl$d(fs, path) {
	return fs.lstatSync(path)
}

async function async_impl$d(fs, path) {
	return await fs.promises.lstat(path)
}

var lstat = {sync_impl: sync_impl$d, async_impl: async_impl$d};

function sync_impl$c(fs, path, options = null) {
	if (options === null) {
		fs.mkdirSync(path);
	} else {
		fs.mkdirSync(path, options);
	}
}

async function async_impl$c(fs, path, options = null) {
	if (options === null) {
		await fs.promises.mkdir(path);
	} else {
		await fs.promises.mkdir(path, options);
	}
}

var mkdir = {sync_impl: sync_impl$c, async_impl: async_impl$c};

function sync_impl$b(fs, path) {
	const handle = fs.opendirSync(path);

	return {
		read() {
			return handle.readSync()
		},
		close() {
			handle.closeSync();
		}
	}
}

async function async_impl$b(fs, path) {
	const handle = await fs.promises.opendir(path);

	return {
		async read() {
			return await handle.read()
		},
		async close() {
			await handle.close();
		}
	}
}

var openDirectory = {sync_impl: sync_impl$b, async_impl: async_impl$b};

function sync_impl$a(fs, path, flags = "r", mode = 0o666) {
	const fd = fs.openSync(path, flags, mode);

	return {
		read(buffer) {
			return fs.readSync(fd, buffer)
		},
		write(buffer) {
			return fs.writeSync(fd, buffer)
		},
		close() {
			fs.closeSync(fd);
		}
	}
}

async function async_impl$a(fs, path, flags = "r", mode = 0o666) {
	const handle = await fs.promises.open(path, flags, mode);

	return {
		async read(buffer) {
			const {bytesRead} = await handle.read(buffer, 0, buffer.byteLength, null);

			return bytesRead
		},
		async write(buffer) {
			const {bytesWritten} = await handle.write(buffer);

			return bytesWritten
		},
		async close() {
			await handle.close();
		}
	}
}

var openFile = {sync_impl: sync_impl$a, async_impl: async_impl$a};

function sync_impl$9(fs, path) {
	return fs.readFileSync(path)
}

async function async_impl$9(fs, path) {
	return await fs.promises.readFile(path)
}

var readFile = {sync_impl: sync_impl$9, async_impl: async_impl$9};

function sync_impl$8(fs, path) {
	return fs.readdirSync(path)
}

async function async_impl$8(fs, path) {
	return await fs.promises.readdir(path)
}

var readdir = {sync_impl: sync_impl$8, async_impl: async_impl$8};

function sync_impl$7(fs, path) {
	return fs.readlinkSync(path)
}

async function async_impl$7(fs, path) {
	return await fs.promises.readlink(path)
}

var readlink = {sync_impl: sync_impl$7, async_impl: async_impl$7};

function sync_impl$6(fs, path) {
	return fs.realpathSync(path)
}

async function async_impl$6(fs, path) {
	return await fs.promises.realpath(path)
}

var realpath = {sync_impl: sync_impl$6, async_impl: async_impl$6};

function sync_impl$5(fs, src, dest) {
	fs.renameSync(src, dest);
}

async function async_impl$5(fs, src, dest) {
	await fs.promises.rename(src, dest);
}

var rename = {sync_impl: sync_impl$5, async_impl: async_impl$5};

function sync_impl$4(fs, path) {
	fs.rmdirSync(path);
}

async function async_impl$4(fs, path) {
	await fs.promises.rmdir(path);
}

var rmdir = {sync_impl: sync_impl$4, async_impl: async_impl$4};

function sync_impl$3(fs, path) {
	return fs.statSync(path)
}

async function async_impl$3(fs, path) {
	return await fs.promises.stat(path)
}

var stat = {sync_impl: sync_impl$3, async_impl: async_impl$3};

function sync_impl$2(fs, target, path) {
	fs.symlinkSync(target, path);
}

async function async_impl$2(fs, target, path) {
	await fs.promises.symlink(target, path);
}

var symlink = {sync_impl: sync_impl$2, async_impl: async_impl$2};

function sync_impl$1(fs, path) {
	fs.unlinkSync(path);
}

async function async_impl$1(fs, path) {
	await fs.promises.unlink(path);
}

var unlink = {sync_impl: sync_impl$1, async_impl: async_impl$1};

function sync_impl$h(fs, path, data) {
	return fs.writeFileSync(path, data)
}

async function async_impl$h(fs, path, data) {
	return await fs.promises.writeFile(path, data)
}

var writeFile = {sync_impl: sync_impl$h, async_impl: async_impl$h};

/* warning: this file was automatically generated */

var methods = {
    chmod,
    chown,
    copyFile,
    lstat,
    mkdir,
    openDirectory,
    openFile,
    readFile,
    readdir,
    readlink,
    realpath,
    rename,
    rmdir,
    stat,
    symlink,
    unlink,
    writeFile,
};

function createFSObject({
	sync = false
} = {}) {
	let fs_object = {};

	for (const method_name in methods) {
		const method = methods[method_name];
		const method_implementation = sync ? method.sync_impl : method.async_impl;

		fs_object[method_name] = (...args) => {

			if (process.env) {
				if ("ANIO_NODE_FS_API_DEBUG" in process.env) {
					console.error(`anio-node-fs-api ${method_name}${sync ? "Sync" : "Async"} with args`, args);
				}
			}

			return method_implementation(fs_implementation, ...args)
		};
	}

	return fs_object
}

function tryStat$1(fs_object, path, fn = "lstat") {
	try {
		return fs_object[fn](path)
	} catch (error) {
		//
		// ENOENT means no such entry
		//
		if (error.code === "ENOENT") {
			return false
		}

		// re-throw every other error
		throw error
	}
}

function sync_impl(fs_object, ...args) {
	const path_to_check = path.join(...args);

	const lstat = tryStat$1(fs_object, path_to_check);

	if (lstat === false) return false

	if (lstat.isSymbolicLink()) {
		const stat = tryStat$1(fs_object, path_to_check, "stat");

		if (stat === false) return "link->broken"
		if (stat.isDirectory()) return "link->dir"

		return "link->file"
	}

	if (lstat.isDirectory()) return "dir"

	return "file"
}

async function tryStat(fs_object, path, fn = "lstat") {
	try {
		return await fs_object[fn](path)
	} catch (error) {
		//
		// ENOENT means no such entry
		//
		if (error.code === "ENOENT") {
			return false
		}

		// re-throw every other error
		throw error
	}
}

async function async_impl(fs_object, ...args) {
	const path_to_check = path.join(...args);

	const lstat = await tryStat(fs_object, path_to_check);

	if (lstat === false) return false

	if (lstat.isSymbolicLink()) {
		const stat = await tryStat(fs_object, path_to_check, "stat");

		if (stat === false) return "link->broken"
		if (stat.isDirectory()) return "link->dir"

		return "link->file"
	}

	if (lstat.isDirectory()) return "dir"

	return "file"
}

const async_fs = createFSObject({sync: false});
const sync_fs = createFSObject({sync: true});

function nodeFsGetPathType(...args) {
	return async_impl(async_fs, ...args)
}

nodeFsGetPathType.sync = function(...args) {
	return sync_impl(sync_fs, ...args)
};

export { nodeFsGetPathType as default };
